# Brandfolder Drupal Integration
-------------------------------------------------------------------------------
![CircleCI](https://img.shields.io/circleci/build/github/brandfolder/integration-drupal?token=94d3945b6680f1ca5e786886ac94757e49c5136b)

This module allows you to manage images using the Brandfolder
(https://brandfolder.com) Digital Asset Management system and reference those
images from your Drupal site. It creates an additional file system in Drupal,
similar to the "public" and "private" file systems, but the Brandfolder file
system is read-only by design. The intended workflow is to upload and curate
images in your Brandfolder portal and then use this module to associate those
images with Drupal content via image fields.


# Dependencies
-------------------------------------------------------------------------------
 - An active Brandfolder account (see https://brandfolder.com) with "Smart CDN"
functionality enabled.
 - The Brandfolder PHP SDK/library (see below).

# Installation
-------------------------------------------------------------------------------
Install the module as you would install any other Drupal module. We recommend 
[using Drush](https://drushcommands.com/drush-8x/pm/pm-enable).

## Important: Composer and Dependencies
The Brandfolder module uses Composer to manage the packages/libraries/etc. that 
it needs in order to function properly. The most important dependency is the
[Brandfolder PHP SDK](https://github.com/orientier/brandfolder-api-php). 
For Drupal 7 sites, the best way to incorporate Composer into your workflow is 
the [Composer Manager](https://www.drupal.org/project/composer_manager) module.
If you are not already using Composer Manager, and you enable the Brandfolder
module using Drush, Drush will prompt you to download/enable the Composer
Manager module. If you encounter an error like "RuntimeException: Autoloader 
not found," this can be resolved as follows:

1. In your terminal, navigate to the `sites/all/files/composer` directory 
   (or wherever you configured Composer Manager to store its `composer.json` 
   file).
2. Run the `composer update` command. This should generate the necessary files, 
   including an `autoload.php` file in the `sites/all/vendor` directory (or 
   wherever you configured Composer Manager to store its vendor files).

# Initial Setup
-------------------------------------------------------------------------------
1. Log in to your Brandfolder account as an administrator
2. Navigate to "My Profile" > "Integrations"
3. In the "API Keys" section, copy the key for the Brandfolder organization you
wish to integrate.
4. Visit /admin/config/media/brandfolder/settings on your Drupal site
5. Enter the API key and save the settings form.

You can also set the key in your site's settings.php file
(e.g. sites/default/settings.php) if desired.
$conf['brandfolder_api_key'] = 'YOUR API KEY';

# Configuration
-------------------------------------------------------------------------------
## Image Fields
Add a field of type Image and set the "File system" to "Brandfolder" in the
"Field Settings" tab.

That field will then function differently than a normal image field.
Specifically:
 - When adding files to a Drupal entity via that field, users will be unable to
   upload new files; rather, they will be presented with a library of
   Brandfolder images from which to choose.
 - When users select a Brandfolder image and save the Drupal entity, Drupal will
   associate the selected Brandfolder image with that Drupal entity, but will
   not copy the file into your web server's file system.
 - When displaying the image, Drupal will deliver an image tag with a URL
   from Brandfolder's "Smart CDN." The Brandfolder module also supports Drupal
   image styles consisting of scaling and/or cropping effects, and does so
   without needing to copy images from Brandfolder to Drupal. Images are
   transformed using Brandfolder's URL transformation API rather than Drupal's
   image toolkit.

## WYSIWYG Editor
If you use the [CKEditor module](https://www.drupal.org/project/ckeditor) to
provide rich text editing capabilities, the Brandfolder module offers a
CKEditor plugin to allow Brandfolder images to be directly inserted into rich
text/HTML fields. You can enable this for any text format as follows:

1. (Optional) On the Brandfolder admin/config page
(admin/config/media/brandfolder), customize the label for the
Brandfolder Browser button that will appear in the CKEditor toolbar. This
defaults to "Insert an Image." You can also specify image transformations
(e.g. scale and crop) to apply to all Brandfolder images inserted via CKEditor.
2. Go to admin/config/content/ckeditor
3. Click the "edit" link for the text format you wish to configure
4. Expand the "Editor appearance" section
5. Drag the Brandfolder Browser button into the toolbar. The button has an image
icon identical to the standard Image button (but if you hover over it, it should
display the text you configured in step 1).
6. (Optional) Remove any button(s) you no longer wish editors to use.
7. Scroll down to the "Plugins" list and check the checkbox next to
"Brandfolder image browser"
8. Save the configuration form


# Uninstallation
-------------------------------------------------------------------------------
It is recommended to disable and uninstall the module via Drush, which will
allow Composer Manager to clean up the module's dependencies.
When you uninstall the module, note that all managed files using the Brandfolder
file scheme will be removed (i.e. any Brandfolder-hosted files you added to your
site using this module). The file records will be removed from Drupal only. No
assets will be removed from Brandfolder.
Before uninstalling, you may wish to reconfigure any image fields that are using
the Brandfolder Browser widget and the Brandfolder file storage location/scheme,
so you can choose how you wish to handle images for those fields going forward.
Otherwise, the module will do its best to clean up these fields, setting their
file storage location to "public" and their widgets to the standard image file
upload widget.

# Acknowledgements
-------------------------------------------------------------------------------
The Amazon S3 File System module (https://www.drupal.org/project/s3fs) served
as a great reference in terms of defining a custom file stream wrapper, etc.
