/**
 * @file
 * Provides default functions for the brandfolder browser
 */

(function ($) {

  Drupal.behaviors.BrandfolderBrowser = {
    attach: function (context) {
      // if (Drupal.settings.brandfolder && Drupal.settings.brandfolder.interestingSetting) {
      // }

      // Relocate all BF browser modals for clean display. They will remain
      // associated with their original context via data attributes/IDs.
      $('.brandfolder-browser-modal').detach().appendTo('body');
      // Add an element to use as a visual modal layer separator and click
      // catcher.
      if ($('.brandfolder-browser-modal-cover').length == 0) {
        var $modal_cover = $('<div class="brandfolder-browser-modal-cover"></div>');
        $modal_cover.unbind('click').bind('click', function () {
          $('.brandfolder-browser-modal').removeClass('modal--active');
          $('body').removeClass('brandfolder-modal--active');
        });
        $modal_cover.appendTo('body');
      }

      $('.brandfolder-browser-element-wrapper .button.browse', context).once().bind('click', function (event) {
        var $button = $(this);
        var $element = $button.closest('.brandfolder-browser-element-wrapper');
        var browser_modal_id = $element.data('browser-modal-id');
        var $browser_modal = $('#' + browser_modal_id);
        $browser_modal.addClass('modal--active');
        $('body').addClass('brandfolder-modal--active');
        event.preventDefault();
      });

      // Handle image selections.
      $('.brandfolder-browser__content').unbind('click').bind('click', function (event) {
        Drupal.brandfolder.browser.handleSelectionEvents(event);
      });

      // @todo consider hijacking tab events to help keep keyboard focus within the modal as long as it's open.

      // Handle search and filter input.
      var $form_wrapper = $('.brandfolder-browser__search-and-filter-form');
      // Clicking the "Search" button.
      $form_wrapper.find('.brandfolder-search__submit-button').unbind('click').bind('click', function (event) {
        event.preventDefault();
        Drupal.brandfolder.browser.getAssets($(this).closest('.brandfolder-browser__search-and-filter-form'));
      });
      // Pressing the "Enter" key from within the search textfield.
      $form_wrapper.find('.brandfolder-search__text').unbind('keydown').bind('keydown', function (event) {
        // Note jQuery backward compatibility.
        if (event.key === 'Enter' || event.keyCode === 13) {
          event.preventDefault();
          Drupal.brandfolder.browser.getAssets($(this).closest('.brandfolder-browser__search-and-filter-form'));
        }
      });
      // Changing any values for filter or sort controls (not including tags).
      $form_wrapper.find('.brandfolder-sort-container, .brandfolder-filters__aspect, .brandfolder-filters__upload-date').unbind('change').bind('change', function () {
        Drupal.brandfolder.browser.getAssets($(this).closest('.brandfolder-browser__search-and-filter-form'));
      });
      // De-selecting tags.
      $form_wrapper.find('.brandfolder-filters__selected-tags').unbind('click').bind('click', function (event) {
        Drupal.brandfolder.browser.handleTagDeselection(event);
      });
      // Entering text in the Tags textfield.
      $form_wrapper.find('.brandfolder-filters__tag-text').unbind('keydown').bind('keydown', function (event) {
        // Note jQuery backward compatibility.
        if (event.key === 'Enter' || event.keyCode === 13) {
          event.preventDefault();
          var $textfield = $(this);
          Drupal.brandfolder.browser.handleTagSelection($textfield);
        }
      });
      // Clicking the "Add" button next to the tags textfield.
      $form_wrapper.find('.brandfolder-filters__tag-add-button').unbind('click').bind('click', function (event) {
        event.preventDefault();
        var $textfield = $(this).closest('.brandfolder-filters__tags-container').find('.brandfolder-filters__tag-text');
        Drupal.brandfolder.browser.handleTagSelection($textfield);
      });

      // Tags autocomplete handling.
      // @todo: See if Brandfolder will support tag search via API. If not, consider maintaining a local store/cache and powering autocomplete with that.
      // $form_wrapper.find('brandfolder-filters__tag-text').unbind('keydown').bind('keydown', function() {
      //   var $textfield = $(this);
      //   var $autocomplete_options_el = $textfield.closest('.brandfolder-filters__tags-container').find('.brandfolder-filters__tag-autocomplete-options');
      //   $autocomplete_options_el.html('').hide();
      //   var text = $textfield.val();
      //   if (text.length > 0) {
      //
      //     $.ajax({
      //       method: "GET",
      //       url: "/brandfolder/get-tags",
      //       data: {
      //         'text': text
      //       }
      //     })
      //       .done(function (response) {
      //         var data = JSON.parse(response);
      //         if (typeof data == 'object' && data.tags) {
      //           var tags = data.tags;
      //           var options = '';
      //           for (var i in tags) {
      //             var tag = tags[i];
      //             options += '<span class="tag-option" data-tag-name="' + tag + '">' + tag + '</span>';
      //           }
      //           $autocomplete_options_el.html(options);
      //           $autocomplete_options_el.unbind('click').bind('click', function(event) {
      //             Drupal.brandfolder.browser.handleTagSelection(event);
      //           });
      //         }
      //       })
      //       .fail(function () {
      //         // @todo.
      //       })
      //       .always(function () {
      //         // @todo.
      //       });
      //   }
      // });

      // Load an initial set of images for each browser.
      $form_wrapper.each(function () {
        Drupal.brandfolder.browser.getAssets($(this));
      });

      // @todo: Pagination/load more/infinite scroll.

      // Temporary JS-only handling for the "Remove" button.
      // @see brandfolder_browser_form_element_update().
      $('.brandfolder-browser-element-wrapper .remove-button').unbind('click').bind('click', function (event) {
        event.preventDefault();
        var $button = $(this);
        var $element = $button.closest('.brandfolder-browser-element-wrapper');
        $element.find('.fid').val(0);
        $element.find('.brandfolder-asset-id').val('');
        $element.find('.preview').html('');
        // @todo: Standard Drupal AJAX submission.
        $element.find('.manualcrop-thumblist').remove();
        $button.addClass('element-invisible');
      });
    }
  };

  Drupal.brandfolder = Drupal.brandfolder || {};
  Drupal.brandfolder.browser = Drupal.brandfolder.browser || {};

  Drupal.brandfolder.browser.handleSelectionEvents = function (event) {
    var selected_asset = event.originalEvent.target;
    var $selectedAsset = $(selected_asset);
    if ($selectedAsset.hasClass('brandfolder-asset__image')) {
      $selectedAsset = $selectedAsset.closest('.brandfolder-asset');
    }
    if ($selectedAsset.hasClass('brandfolder-asset')) {
      var asset_id = $selectedAsset.data('bf-asset-id');
      var $modal = $selectedAsset.closest('.brandfolder-browser-modal');
      var element_id = $modal.data('triggering-element');
      var $element = $('#' + element_id);
      if ($element.length > 0) {
        $element.find('.brandfolder-asset-id').val(asset_id);
        $element.find('.preview').html(selected_asset.outerHTML);
        $element.find('.image-selection-submit').trigger('mousedown');
        $element[0].scrollIntoView();
      }
      $modal.removeClass('modal--active');
      $('body').removeClass('brandfolder-modal--active');
    }
  };

  // @todo: See if Brandfolder will support tag search via API. If not, consider maintaining a local store/cache and powering autocomplete with that.
  // Drupal.brandfolder.browser.handleTagSelection = function(event) {
  //   var $selected_el = $(event.originalEvent.target);
  //   var tag_name = $selected_el.data('tag-name');
  //   if (tag_name) {
  //     var $tag_container = $selected_el.closest('.brandfolder-filters__tags-container');
  //     // Reset the autocomplete elements.
  //     $selected_el.closest('.brandfolder-filters__tag-autocomplete-options').html('').hide();
  //     $tag_container.find('.brandfolder-filters__tag-text').val('');
  //     // Add an item to the displayed list of selected tags, unless the selected
  //     // tag happens to already be in the list.
  //     var $tag_list = $tag_container.find('.brandfolder-filters__selected-tags');
  //     var duplicate = false;
  //     for (var i = 0; i < $tag_list.length; i++) {
  //       var existing_tag = $tag_list[i];
  //       if ($(existing_tag).data('tag-name') == tag_name) {
  //         duplicate = true;
  //         break;
  //       }
  //     }
  //     if (!duplicate) {
  //       $tag_list.append('<span class="selected-tag" data-tag-name="' + tag_name + '">' + tag_name + '</span>');
  //       Drupal.brandfolder.browser.getAssets($tag_container.closest('.brandfolder-browser__search-and-filter-form'));
  //     }
  //   }
  // };

  Drupal.brandfolder.browser.handleTagSelection = function ($tag_textfield) {
    var tag_name = $tag_textfield.val();
    if (tag_name.length > 0) {
      $tag_textfield.val('');
      // Add an item to the displayed list of selected tags, unless the selected
      // tag happens to already be in the list.
      var $tag_container = $tag_textfield.closest('.brandfolder-filters__tags-container');
      var $tag_list = $tag_container.find('.brandfolder-filters__selected-tags');
      var $existing_tags = $tag_list.children();
      var duplicate = false;
      for (var i = 0; i < $existing_tags.length; i++) {
        var existing_tag = $existing_tags[i];
        if ($(existing_tag).data('tag-name') == tag_name) {
          duplicate = true;
          break;
        }
      }
      if (!duplicate) {
        $tag_list.append('<span class="selected-tag" data-tag-name="' + tag_name + '"><span class="selected-tag__name">' + tag_name + '</span><span class="selected-tag__remove">x</span></span>');
        Drupal.brandfolder.browser.getAssets($tag_container.closest('.brandfolder-browser__search-and-filter-form'));
      }
    }
  };

  Drupal.brandfolder.browser.handleTagDeselection = function (event) {
    var $selected_el = $(event.originalEvent.target);
    if ($selected_el.hasClass('selected-tag__remove')) {
      var $tag = $selected_el.closest('.selected-tag');
      if ($tag.length > 0) {
        var tag_name = $tag.data('tag-name');
        if (tag_name) {
          var $form_wrapper = $tag.closest('.brandfolder-browser__search-and-filter-form');
          $tag.remove();
          Drupal.brandfolder.browser.getAssets($form_wrapper);
        }
      }
    }
  };

  Drupal.brandfolder.browser.getAssets = function ($form_wrapper) {
    // Collect search/sort/filter input and compile AJAX request.
    var params = {};
    var context_id = params['context'] = $form_wrapper.find('.brandfolder-browser-context-id').val();
    params['search'] = $form_wrapper.find('.brandfolder-search__text').val();
    params['sort'] = $form_wrapper.find('.brandfolder-sort__criterion').val();
    params['order'] = $form_wrapper.find('.brandfolder-sort__order').val();
    var orientation = [];
    $form_wrapper.find('.brandfolder-filters__aspect:checked').each(function () {
      orientation.push($(this).val());
    });
    if (orientation.length > 0) {
      params['orientation'] = orientation.join(',');
    }
    var $upload_date_selection = $form_wrapper.find('.brandfolder-filters__upload-date:checked');
    if ($upload_date_selection.length > 0) {
      var upload_date_val = $upload_date_selection.val();
      if (upload_date_val != 'all') {
        params['created_at'] = upload_date_val;
      }
    }

    // Tags.
    var tags = [];
    var tag_elements = $form_wrapper.find('.brandfolder-filters__selected-tags').children();
    for (var i = 0; i < tag_elements.length; i++) {
      var tag_name = $(tag_elements[i]).data('tag-name');
      if (tag_name) {
        tags.push(tag_name);
      }
    }
    if (tags.length > 0) {
      params['tags'] = tags.join(',');
    }

    // Additional settings.
    if (Drupal.settings.brandfolder && Drupal.settings.brandfolder.element_data && Drupal.settings.brandfolder.element_data[context_id]) {
      var settings = Drupal.settings.brandfolder.element_data[context_id];
      var available_settings = [
        'allowed_file_extensions',
        'max_file_size',
        'min_resolution',
        'max_resolution',
      ];
      for (var i in available_settings) {
        var setting_name = available_settings[i];
        if (typeof settings[setting_name] != 'undefined') {
          params[setting_name] = settings[setting_name];
        }
      }
    }

    var $search_result_container = $form_wrapper.closest('.brandfolder-browser-modal').find('.brandfolder-browser__content');
    $search_result_container.addClass('search-in-progress');

    // This AJAX request and response handling is meant to be compatible with
    // jQuery version 1.4.4 all the way through to 3.x.

    $.ajax({
      method: "GET",
      url: "/brandfolder/get-assets",
      data: params,
      success: function (responseData, status, request) {
        if (responseData.length > 0) {
          var data = JSON.parse(responseData);
          if (typeof data == 'object' && data.content) {
            $search_result_container.html(data.content);

            return;
          }
        }
        // Display a generic error message if we did not receive meaningful
        // response data.
        $search_result_container.html('There was a problem searching for images');
      },
      error: function (request, errorType) {
        $search_result_container.html('There was a problem searching for images');
      },
      complete: function (request, status) {
        $search_result_container.removeClass('search-in-progress');
        // Add an image for indicating that a search is in-progress.
        if (Drupal.settings.brandfolder && Drupal.settings.brandfolder.loading_indicator_path) {
          // @todo: Better method.
          $search_result_container.append('<div class="loading-indicator"> <img src="/' + Drupal.settings.brandfolder.loading_indicator_path + '" class="loading-indicator__image" > </div>');
        }
      }
    });
  }

}(jQuery));
