(function ($) {
  CKEDITOR.plugins.add( 'brandfolder_browser', {
    init: function ( editor ) {
      editor.addCommand( 'brandfolderBrowseImages', {
        exec : function ( editor ) {
          var $bf_browser_element_wrapper = $(editor.element.$).closest('.brandfolder-browser-element-wrapper');
          if ($bf_browser_element_wrapper.length > 0) {
            var modal_id = $bf_browser_element_wrapper.data('browser-modal-id');
            var $browser_modal = $('#' + modal_id);
            $browser_modal.addClass('modal--active');
            $('body').addClass('brandfolder-modal--active');
            $browser_modal.find('.brandfolder-browser__content').unbind('click').bind('click', function (event) {
              var selected_asset = event.originalEvent.target;
              var $selected_asset = $(selected_asset);
              if ($selected_asset.hasClass('brandfolder-asset__image')) {
                var $img = $selected_asset;
              }
              else {
                if ($selected_asset.hasClass('brandfolder-asset')) {
                  var $img = $selected_asset.find('img');
                }
              }
              if (typeof $img != 'undefined' && $img.length > 0) {
                var cdn_url = $img.data('bf-asset-cdn-url');
                // The CDN URL provided by the Brandfolder API often defaults to
                // a .png extension. Try to ensure the most appropriate
                // extension is used.
                // @todo Allow more configurability/alteration here.
                // @todo Possibly switch to a purely token-based approach.
                var extension = null;
                if ($img.attr('src').match(/\.gif/)) {
                  extension = '.gif';
                }
                else {
                  if (Drupal.settings.brandfolder.cdn.cdn_url_static_img_format.length > 0) {
                    extension = '.' + Drupal.settings.brandfolder.cdn.cdn_url_static_img_format;
                  }
                }
                if (extension) {
                  cdn_url = cdn_url.replace(/\.(png|jpg|jpeg|webp|auto)$/, extension);
                }
                else {
                  var match = cdn_url.match(/\.\w+$/);
                  if (match) {
                    extension = match[0].toLowerCase();
                  }
                  else {
                    extension = '.jpg';
                  }
                }

                // Immediately insert the existing thumbnail image into the
                // editor. We will later load the final, full-size image from
                // the CDN asynchronously and use it to replace the low-res
                // thumbnail once it's available.
                // Note: it is possible for users to save the host entity
                // while we're still waiting for the CDN image to load, in
                // which case the low-res image will be saved.
                // @todo: While this seems unlikely, consider ways to safeguard against it. We could modify the placeholder image in a way that makes it obvious to users that it's a placeholder, or prevent them from saving the host entity until it's been replaced, but those have their own drawbacks.
                var temp_img = $img[0].outerHTML;
                var matches = temp_img.match(/src="([^"]+)"/);
                var imgElement = new CKEDITOR.dom.element('img');
                var imgAttributes = {
                  'src': matches[1],
                  'alt': $img.attr('alt')
                };
                // Apply any transformations configured via BF module admin.
                if (Drupal.settings.brandfolder.rte.image_url_params.length > 0) {
                  var params = Drupal.settings.brandfolder.rte.image_url_params;
                  var match = params.match(/^([^&]+&)?width=(\d+)/);
                  if (match) {
                    imgAttributes['width'] = match[2];
                  }
                  if (extension.match(/\.(png|jpg|jpeg|png|webp|auto)/)) {
                    cdn_url = cdn_url + '?' + params;
                  }
                }
                imgElement.setAttributes(imgAttributes);
                imgElement.data('bf-asset-cdn-url', cdn_url);
                imgElement.data('bf-asset-id', $selected_asset.data('bf-asset-id'));

                editor.insertElement(imgElement);

                var $modal = $selected_asset.closest('.brandfolder-browser-modal');
                $modal.removeClass('modal--active');
                $('body').removeClass('brandfolder-modal--active');
              }
            });
          }
        }
      });

      var bf_browser_button_label = 'Insert an image';
      if (Drupal.settings.brandfolder.rte.bf_browser_button_label.length > 0) {
        bf_browser_button_label = Drupal.settings.brandfolder.rte.bf_browser_button_label;
      }
      editor.ui.addButton('brandfolderBrowserButton', {
        label : bf_browser_button_label,
        command: 'brandfolderBrowseImages',
        icon : this.path + 'images/image_icon.png'
      });

      editor.on('insertElement', function (event) {
        var element = event.data;
        if (element.getName() === 'img' && element.hasAttribute('data-bf-asset-cdn-url')) {
          var cdn_url = element.data('bf-asset-cdn-url');

          var fetchFinalImg = new Promise(function (resolve, reject) {
            var img = new Image();
            img.src = cdn_url;
            img.onload = function () {
              resolve(img);
            }
          });

          fetchFinalImg.then(function (img) {
            if (img) {
              var betterImgElement = new CKEDITOR.dom.element('img');
              betterImgElement.setAttributes({
                'src': cdn_url,
                'class': 'brandfolder-asset',
                'alt': element.getAttribute('alt')
              });
              betterImgElement.data('bf-asset-id', element.data('bf-asset-id'));
              // Replace the low-res image with the proper CDN image.
              // Note: we insert the new element and then remove the old one,
              // because element.replace(betterImgElement) does not work here
              // since element has not been fully added to the DOM oslt.
              element.insertBeforeMe(betterImgElement);
              element.remove();
            }
          });
        }
      }, null, null, 1);
    }
  });
})(jQuery);
